%% Model coupling FitzHugh-Nagumo with Allen-Cahn -> interface excitations 
% hannes.uecker@uni-oldenburg.de, 2011/04  
% uses some 'subroutine' anno1(title) to set fonts etc. 
%
% IC and discretization 
npde=3; nx=200; ny=200; L=15*pi; dx=2*L/nx; dy=2*L/ny; 
x=-L:dx:L-dx; y=-L:dy:L-dy; [X Y]=meshgrid(x,y);t=0; 
X=X'; Y=Y'; uv=zeros(npde,nx,ny); 
uhom=-0.58; vhom=-0.38; t=0; 
u0=uhom-0*(uhom-0.1)*(X.^2+Y.^2<2/L); v0=vhom*ones(nx,ny); 
q0=zeros(nx,ny); qs1=1; qs2=-qs1;
qxp=L/4; qyp=0; %AC-IC, wedge-like
q0=1-2*((X<qxp+qs1*(Y-qyp)).*(Y<qyp)+(X<qxp+qs2*(Y-qyp)).*(Y>=qyp)); 
uv(1,:,:)=u0; uv(2,:,:)=v0; uv(3,:,:)=q0; 
figure(1);clf; pcolor(X,Y,u0);colorbar;grid off; colormap gray; 
shading interp; tit=title(['u,  t= ',num2str(t)]); anno1(tit,L);
set(figure(1),'Position', [620 400 370 320]);
figure(2);clf; pcolor(X,Y,v0);colorbar;grid off; colormap gray; 
shading interp; tit=title(['v,  t= ',num2str(t)]); anno1(tit,L);
set(figure(2),'Position', [1000 400 370 320]);
figure(3);clf; pcolor(X,Y,q0);colorbar;grid off; colormap gray; 
shading interp; tit=title(['q,  t= ',num2str(t)]); anno1(tit,L);
set(figure(3),'Position', [1000 20 370 320]);
%% Parameters 
du=0.3^2; dv=0.1^2; duq=0.1; gamma=-0.05; a=-0.1; 
al=1; beta=0.2; del=0.5; epsi=0.03; 
eta=0.75*sqrt(2)*a*sqrt(1+qs1^2)%, eta=0
para=zeros(1,100); npara=13; bcp=qxp-qs1*L
para(1:13)=[del a epsi beta al gamma du dv duq uhom vhom bcp eta]; 
node=npde*nx*ny; lpw=1; lpwi=(3*(nx+1)*npde-2)*node; MORDER=5; 
LU=node*(MORDER+1); nwork=npde*(npde*(nx*2+3)+13+nx)+nx*4+4*node+lpw+LU
nworki=npde*(npde*(nx*2+3)+13+nx)+nx*4+4*node+lpwi+LU
%% first integration
t0=0; t1=10; locerr=0.0001; h=0.001; 
mf=10; index=1; !mf=22; 
tic;[uv, index, h]=chemstep(t0,t1,h,locerr,mf,index,x,y,uv,para,npara); toc
h,u=squeeze(uv(1,:,:)); v=squeeze(uv(2,:,:)); q=squeeze(uv(3,:,:)); 
figure(3);clf;pcolor(X,Y,q);colorbar; shading interp; 
grid off; tit=title(['q,  t= ',num2str(t1)]); anno1(tit,L); 
%%  Integration Loop 
dt=t1-t0;dt=asknu('dt (0 for end)',dt); isteps=9; 
while(dt>0)
    isteps=asknu('isteps',isteps);ic=0; 
    while(ic<isteps) 
    t0=t1; t1=t0+dt; 
    tic;[uv, index, h]=chemstep(t0,t1,h,locerr,mf,index,x,y,uv,para,npara); toc
    h, u=squeeze(uv(1,:,:)); v=squeeze(uv(2,:,:)); q=squeeze(uv(3,:,:));
    figure(1);clf;pcolor(X,Y,real(u));colorbar; shading interp; 
    grid off; tit=title(['u,  t= ',num2str(t1)]); anno1(tit,L); 
    figure(2);clf;pcolor(X,Y,real(v));colorbar; shading interp; 
    grid off; tit=title(['v, t= ',num2str(t1)]); anno1(tit,L);
    figure(3);clf;pcolor(X,Y,q);colorbar; shading interp; 
    grid off; tit=title(['q,  t= ',num2str(t1)]); anno1(tit,L); 
    ic=ic+1; 
    end
    dt=asknu('dt (0 for end)',dt);
end 