! model coupling FHN and Allen-Cahn to yield excited fronts
! hannes.uecker@uni-oldenburg.de, 2011/04  
!
! compile as    mex chemstep.F pde2d.o, see chemdr.m for more 
#include "fintrf.h"
      subroutine mexFunction(nlhs, plhs, nrhs, prhs)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      parameter (nx=200, ny=200, npde=3, nwork=10000000)     
      mwpointer mxCreateDoubleMatrix,mxGetPr,mxCreateNumericMatrix
      double precision mxGetScalar
      integer prhs(*),plhs(*) 
      mwpointer x_pr, y_pr, u_pr, para_pr, o1_pr, o_pr2, o_pr3
      dimension X(nx),Y(ny), U(npde,nx,ny), WORK(nwork), para(100) 
      integer IWORK(500000) 
      dimension ndims(3)     ! to pass dimensions back to matlab 
      character(200) mstring ! for printing info 
      common /hupara/ para   ! parameters to pass to PDE and BC 
      t0=mxGetScalar(prhs(1)); tout=mxGetScalar(prhs(2)); 
      h=mxGetScalar(prhs(3)); eps=mxGetScalar(prhs(4)); 
      mf=mxGetScalar(prhs(5));index=mxGetScalar(prhs(6));
      npara=mxGetScalar(prhs(11));
      node=npde*nx*ny; 
      x_pr=mxGetPr(prhs(7));call mxCopyPtrToReal8(x_pr,x,nx); ! x-input
      y_pr=mxGetPr(prhs(8));call mxCopyPtrToReal8(y_pr,y,ny); ! y-input
      u_pr=mxGetPr(prhs(9));call mxCopyPtrToReal8(u_pr,u,node); ! y-input 
      para_pr=mxGetPr(prhs(10));
      call mxCopyPtrToReal8(para_pr,para,npara); ! fill common 
      LPW=(3*(NX+1)*NPDE-2)*NODE; if(mf.lt.21) lpw=1; 
      MORDER=5; LU=NODE*(MORDER+1);
      nworkm=NPDE*(NPDE*(NX*2+3)+13+NX)+NX*4+4*NODE+LPW+LU; 
      if(nworkm.gt.nwork) then
        write(mstring,*)  'nwork=',nworkm,'> max=',nwork,  achar(13), 
     1    'Try MF=10' 
        k=mexPrintf(mstring); goto 990
      endif
      IWORK(1)= NPDE; IWORK(2) = NX; IWORK(3) = NY; IWORK(4) = MORDER
      IWORK(5)=nworkm;   IWORK(6)=NODE
      write(mstring,*) ' calling pde2 ... ', achar(13); 
      k=mexPrintf(mstring); k=mexEvalString("drawnow;"); 
      CALL DRIVEP(NODE,T0,H,U,TOUT,EPS,MF,INDEX,WORK,IWORK,X,Y)
      write(mstring,*) '... back, t1=', tout, index, achar(13); 
      k=mexPrintf(mstring)
 990  ndims(1)=npde; ndims(2)=nx; ndims(3)=ny; ! pass back to matlab 
      plhs(1)=mxCreateNumericArray(3, ndims, 
     *          mxClassIDFromClassName('double'), 0) 
      o1_pr=mxGetPr(plhs(1)); 
      call mxCopyReal8ToPtr(u,o1_pr,node); ! u
      plhs(2)=mxCreateDoubleMatrix(1,1,0); o_pr2=mxGetPr(plhs(2)); 
      call mxCopyInteger1ToPtr(index, o_pr2, 1)  ! index
      plhs(3)=mxCreateDoubleMatrix(1,1,0); o_pr3=mxGetPr(plhs(3)); 
      call mxCopyReal8ToPtr(h, o_pr3, 1)  ! h=last stepsize 
 999  return
      end 
! ------------ SUBROUTINES as required by pdetwo ------------------------
      SUBROUTINE BNDRYV (T,X,Y,U,AV,BV,CV,NPDE) ! VERTICAL BC
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION U(NPDE),AV(NPDE),BV(NPDE),CV(NPDE),para(100)
      common /hupara/ para 
      uhom=para(10); vhom=para(11); 
      AV(1)=1.0; BV(1)=0.0; CV(1)=uhom 
      AV(2)=1.0; BV(2)=0.0; CV(2)=vhom 
      av(3)=0.0; BV(3)=1.0; CV(3)=0.d0; ! Neumann 
      RETURN
      END

      SUBROUTINE BNDRYH (T,X,Y,U,AH,BH,CH,NPDE) ! HORIZONTAL BC
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION U(NPDE),AH(NPDE),BH(NPDE),CH(NPDE), para(100)
      common /hupara/ para 
      uhom=para(10); vhom=para(11); bcp=para(12); 
      AH(1)=1.0; BH(1)=0.0; CH(1)=uhom 
      AH(2)=1.0; BH(2)=0.0; CH(2)=vhom
      AH(3)=1.0; BH(3)=0.0; CH(3)=dtanh((x-bcp)/dsqrt(2.d0)); ! Dirichlet 
      RETURn
      END

      SUBROUTINE DIFFH (T,X,Y,U,DH,NPDE) ! HORIZONTAL DIFFUSION COEFFICIENTS
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION U(NPDE),DH(NPDE,NPDE), para(100)
      common /hupara/ para 
      DH(1,1)=1; DH(1,2)=0; DH(1,3)=0; 
      DH(2,1)=0.0; DH(2,2)=1.0; DH(2,3)=0.0
      DH(3,1)=0.0; DH(3,2)=0.0; DH(3,3)=1.0
      RETURN
      END

      SUBROUTINE DIFFV (T,X,Y,U,DV,NPDE) ! VERTICAL DIFFUSION COEFFICIENTS
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION U(NPDE),DV(NPDE,NPDE), para(100)
      common /hupara/ para 
      DV(1,1)=1; DV(1,2)=0; DV(1,3)=0; 
      DV(2,1)=0.0; DV(2,2)=1.0; DV(2,3)=0.0
      DV(3,1)=0.0; DV(3,2)=0.0; DV(3,3)=1.0
      RETURN
      END

      SUBROUTINE F(T,X,Y,Uv,UX,UY,DUXX,DUYY,DUDT,NPDE) !PDE
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION Uv(NPDE),UX(NPDE),UY(NPDE),DUXX(NPDE,NPDE),
     * DUYY(NPDE,NPDE),DUDT(NPDE), para(100)
      character(200) mstring
      common /hupara/ para
      del=para(1); a=para(2); epsi=para(3); beta=para(4); al=para(5)
      ga=para(6); du=para(7); dv=para(8); duq=para(9); uhom=para(10); 
      eta=para(13); 
      u=uv(1); v=uv(2); q=uv(3);  dlapu=DUXX(1,1)+DUYY(1,1); 
      dlapv=DUXX(2,2)+DUYY(2,2);  dlapq=DUXX(3,3)+DUYY(3,3); 
      DUDT(1)=u-u**3-v-del*(u-uhom)*q**2+du*dlapu+duq*dlapq+eta*ux(1); 
      DUDT(2)=epsi*(u+beta-v)+dv*dlapv+eta*ux(2); 
      DUDT(3)=(1-q)*(q-a)*(q+1)+ga*(1-q**2)*(u-uhom)+duq*dlapu
     *        +1.5*dlapq+eta*ux(3); 
      RETURN; END



